# -*- coding: utf-8 -*-
"""
Created on Fri Feb 24 11:32:10 2017

@author: Ulrich Wilbrandt
"""
import meshio
import medit_io
import os
import argparse


def get_points_and_edges_from_mesh(mesh_file):
    points, cells, point_data, cell_data, field_data = medit_io.read(mesh_file)
    # only need points and cells here
    del point_data, field_data
    # do some checks
    if points[0].size is not 2:
        if all(z[2] == 0. for z in points):
            points = [[x, y] for [x, y, z] in points]
    # build all edges
    # this is a set so that each edge only appears once, also each edge is a
    # tuple rather than a list, lists can not be used in a set
    edges = set()
    if 'triangle' in cells:
        for c in cells['triangle']:
            edges.add(tuple(sorted([c[0], c[1]])))
            edges.add(tuple(sorted([c[1], c[2]])))
            edges.add(tuple(sorted([c[0], c[2]])))
    if 'quad' in cells:
        for c in cells['quad']:
            edges.add(tuple(sorted([c[0], c[1]])))
            edges.add(tuple(sorted([c[1], c[2]])))
            edges.add(tuple(sorted([c[2], c[3]])))
            edges.add(tuple(sorted([c[0], c[3]])))
    if 'tetra' in cells:
        for c in cells['tetra']:
            edges.add(tuple(sorted([c[0], c[1]])))
            edges.add(tuple(sorted([c[0], c[2]])))
            edges.add(tuple(sorted([c[0], c[3]])))
            edges.add(tuple(sorted([c[1], c[2]])))
            edges.add(tuple(sorted([c[1], c[3]])))
            edges.add(tuple(sorted([c[2], c[3]])))

    return points, list(edges), cells, cell_data


def read_mesh_file(mesh_file, outfile=None):
    points, edges, cells, cell_data = get_points_and_edges_from_mesh(mesh_file)
    if(outfile is None):
        outfile = os.path.splitext(mesh_file)[0] + '.include.tex'

    dim = 2 if len(points[0]) is 2 else 3

    def node_name(i):
        return "p" + str(i)
    f = open(outfile, 'w')
    f.write("% this file has been gerated by\n")
    f.write("% python3 mesh_to_tikz.py {}\n".format(mesh_file))
    f.write("% Put it into a 'tikzpicture' environment in LaTex ")
    f.write("(e.g. via input).\n")

    if dim is 2:
        for i, [x, y] in enumerate(points):
            f.write("\coordinate ({}) at ({},{});\n".format(node_name(i), x,
                                                            y))
    else:
        for i, [x, y, z] in enumerate(points):
            f.write("\coordinate ({}) at ({},{},{});\n".format(node_name(i), x,
                                                               y, z))
    for e in edges:
        f.write("\draw[very thin] ({}) -- ({});\n".format(node_name(e[0]),
                                                          node_name(e[1])))
    f.close()


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Read a mesh file and write a'
                                     'file which includes tikz commands to '
                                     'draw the mesh.')
    parser.add_argument('mesh_file', help='the mesh file')
    parser.add_argument('-o', '--outfile', default=None,
                        help='file to put the tikz code in')
    parser.add_argument('-c', '--color', help="color the cells",
                        action='store_true')
    args = parser.parse_args()
    read_mesh_file(args.mesh_file, args.outfile)
